"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schema_1 = require("./schema");
const utility_1 = require("../utility");
class MongoDBUserRoleAdapter {
    constructor(db) {
        this.userRoles = db.collection(schema_1.CollectionName.UserRoles);
    }
    async createUserRole(args) {
        const { insertedId: id } = await this.userRoles.insertOne({
            createdAt: new Date(),
            modifiedAt: new Date(),
            name: args.name,
            description: args.description || '',
            systemRole: false,
            permissionIDs: args.permissionIDs
        });
        const userRole = await this.userRoles.findOne({ _id: id });
        if (!userRole) {
            throw new Error('Could not create UserRole');
        }
        return {
            id: userRole._id,
            name: userRole.name,
            description: userRole.description,
            systemRole: userRole.systemRole,
            permissionIDs: userRole.permissionIDs
        };
    }
    async getUserRole(name) {
        const userRole = await this.userRoles.findOne({ name });
        if (userRole) {
            return {
                id: userRole._id,
                name: userRole.name,
                description: userRole.description,
                systemRole: userRole.systemRole,
                permissionIDs: userRole.permissionIDs
            };
        }
        else {
            return null;
        }
    }
    async getUserRoleByID(id) {
        const userRole = await this.userRoles.findOne({ _id: id });
        if (userRole) {
            return {
                id: userRole._id,
                name: userRole.name,
                description: userRole.description,
                systemRole: userRole.systemRole,
                permissionIDs: userRole.permissionIDs
            };
        }
        else {
            return null;
        }
    }
    async getUserRolesByID(ids) {
        const userRoles = await this.userRoles.find({ _id: { $in: ids } }).toArray();
        return userRoles.map(userRole => ({
            id: userRole._id,
            name: userRole.name,
            description: userRole.description,
            systemRole: userRole.systemRole,
            permissionIDs: userRole.permissionIDs
        }));
    }
    async getNonOptionalUserRolesByID(ids) {
        const roles = await this.getUserRolesByID(ids);
        return roles ? roles.filter(utility_1.isNonNull) : [];
    }
}
exports.MongoDBUserRoleAdapter = MongoDBUserRoleAdapter;
//# sourceMappingURL=userRole.js.map